//Libraries
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27,20,4);  //LCD i2c adress sometimes 0x3f or 0x27


//Constants
#define LCD_refresh_rate 1000

//Inputs
#define solar_current_in A0

//Outputs
#define PWM_out 10
#define load_enable 2

//Variables
int pwm_value = 0;
float solar_current = 0;
float current_factor = 0.185;       //Value defined by manufacturer ACS712 5A

unsigned int before_millis = 0;
unsigned int now_millis = 0;



void setup(){  
  pinMode(solar_current_in,INPUT);    //Set pins as INPUTS
  
  pinMode(PWM_out,OUTPUT);            //Set pins as OUTPUTS
  digitalWrite(PWM_out,LOW);          //Set PWM to LOW so MSOFET is off 
  
  TCCR1B = TCCR1B & B11111000 | B00000001; //timer 1 PWM frequency of 31372.55 Hz
  Serial.begin(9600);

  lcd.init();                 //Init the LCD
  lcd.backlight();            //Activate backlight  

  before_millis = millis;     //Used for LCD refresh rate
}

void loop(){ 
  solar_current = get_solar_current(10); 

  now_millis = millis();
  if(now_millis - before_millis > LCD_refresh_rate)
  {
    before_millis = now_millis;
    //lcd.clear();
    lcd.setCursor(0,0);               //Column 0 row 0
    lcd.print(solar_current,2);       //Soalr voltage 
    lcd.print("A");                   //Volts
  }

  ///////////////////////////////////////////////
  //CONSTANT CURRENT CONTROL//    
  if(solar_current > 1.0){                                    //Constant 1A current
    pwm_value--;
    pwm_value = constrain(pwm_value,0,254);
  }
  
  else {
    pwm_value++;
    pwm_value = constrain(pwm_value,0,254);
  }  
  analogWrite(PWM_out,pwm_value);
  ///////////////////////////////////////////////
  ///////////////////////////////////////////////
  
}//End void loop



/////////////////////////FUNCTIONS/////////////////////////
///////////////////////////////////////////////////////////
float get_solar_current(int n_samples)
{
  float Sensor_voltage;
  float current =0;
  for(int i=0; i < n_samples; i++)
  {
    Sensor_voltage = analogRead(solar_current_in) * (5.0 / 1023.0);
    current = current + (Sensor_voltage-2.5)/current_factor; 
  }
  current = current/n_samples;
  return(current);
}
